@@ -1,10 +1,18 @@
-from nonebot import on_command, get_driver
-from nonebot.adapters.onebot.v11 import Bot, Message, MessageSegment, PrivateMessageEvent, GroupMessageEvent, GROUP, GROUP_ADMIN, GROUP_OWNER
+from nonebot import get_bot, on_command, get_driver, require
+from nonebot.adapters.onebot.v11 import Bot, Message, MessageSegment, PrivateMessageEvent, GroupMessageEvent, GROUP_ADMIN, GROUP_OWNER
+from nonebot_plugin_guild_patch import GuildMessageEvent, GUILD_ADMIN, GUILD_OWNER, GUILD_SUPERUSER
 from nonebot.params import CommandArg
 from nonebot.plugin import PluginMetadata
 from nonebot.permission import SUPERUSER
+import sqlite3
+
+require('nonebot_plugin_apscheduler')
+require("nonebot_plugin_localstore")
+
 from nonebot_plugin_apscheduler import scheduler
+import nonebot_plugin_localstore as store
 from nonebot_plugin_txt2img import Txt2Img
+from pathlib import Path
 from httpx import AsyncClient
 from typing import Union
 from .config import Config
@@ -15,15 +23,18 @@ __plugin_meta__ = PluginMetadata(
     'Apex Legends API 查询插件',
     '''
     `/bridge [玩家名称]` 、`/玩家 [玩家名称]` - 根据玩家名称查询玩家信息 (暂仅支持查询 PC 平台玩家信息), 
-    `/uid [玩家UID]`、`/UID [玩家UID]` - 根据玩家 UID 查询玩家信息 (暂仅支持查询 PC 平台玩家信息), 
+    `/uid [玩家 UID]`、`/UID [玩家 UID]` - 根据玩家 UID 查询玩家信息 (暂仅支持查询 PC 平台玩家信息), 
+    `/uid`、`/自查` - 根据玩家已绑定的 UID 自动查询玩家信息, 
     `/maprotation` 、 `/地图` - 查询当前地图轮换, 
     `/predator` 、 `/猎杀` - 查询顶尖猎杀者信息, 
     `/crafting` 、 `/制造` - 查询当前制造轮换, 
     `/servers`、`/服务` - 查看当前服务器状态, 
-    `/submap`、`/订阅地图` - 订阅地图轮换(每整点查询)(仅群聊可用), 
-    `/unsubmap`、`/取消订阅地图` - 取消订阅地图轮换(仅群聊可用), 
-    `/subcraft`、`/订阅制造` - 订阅制造轮换(每日2时查询)(仅群聊可用), 
-    `/unsubcraft`、`/取消订阅制造` - 取消订阅制造轮换(仅群聊可用).
+    `/submap`、`/订阅地图` - 订阅地图轮换 (每整点查询) (仅 群聊/频道 可用), 
+    `/unsubmap`、`/取消订阅地图` - 取消订阅地图轮换 (仅 群聊/频道 可用), 
+    `/subcraft`、`/订阅制造` - 订阅制造轮换 (每日 2 时查询) (仅 群聊/频道 可用), 
+    `/unsubcraft`、`/取消订阅制造` - 取消订阅制造轮换 (仅 群聊/频道 可用), 
+    `/bind [玩家 UID]`、`/绑定 [玩家 UID]` - 将 UID 与 QQ 账号绑定, 
+    `/unbind`、`/解绑` - 将 UID 与 QQ 账号解除绑定.
     '''
 )
 
@@ -32,87 +43,110 @@ plugin_config = Config.parse_obj(get_driver().config)
 api_key = plugin_config.apex_api_key
 api_url = plugin_config.apex_api_url
 api_t2i = plugin_config.apex_api_t2i
+plugin_data_file: Path = store.get_data_file("nonebot_plugin_apex_api_query", "database.db")
 
 # 创建事件响应器
-player_statistics = on_command('bridge', aliases= {'玩家'})
-uid_statistics = on_command('uid', aliases={'UID'})
-map_protation = on_command('maprotation', aliases={'地图'})
-predator = on_command('predator', aliases={'猎杀'})
-crafting_rotation = on_command('crafting', aliases={'制造'})
-servers = on_command('servers', aliases={'服务'})
-sub_map = on_command('submap', aliases={'订阅地图'}, permission=GROUP and SUPERUSER | GROUP_ADMIN | GROUP_OWNER)
-unsub_map = on_command('unsubmap', aliases={'取消订阅地图'}, permission=GROUP and SUPERUSER | GROUP_ADMIN | GROUP_OWNER)
-sub_craft = on_command('subcraft', aliases={'订阅制造'}, permission=GROUP and SUPERUSER | GROUP_ADMIN | GROUP_OWNER)
-unsub_craft = on_command('unsubcraft', aliases={'取消订阅制造'}, permission=GROUP and SUPERUSER | GROUP_ADMIN | GROUP_OWNER)
+player_statistics = on_command('bridge', aliases = {'玩家'})
+uid_statistics = on_command('uid', aliases = {'UID', '自查'})
+map_protation = on_command('maprotation', aliases = {'地图'})
+predator = on_command('predator', aliases = {'猎杀'})
+crafting_rotation = on_command('crafting', aliases = {'制造'})
+servers = on_command('servers', aliases = {'服务'})
+sub_map = on_command('submap', aliases = {'订阅地图'}, permission = SUPERUSER | GROUP_ADMIN | GROUP_OWNER | GUILD_ADMIN | GUILD_OWNER | GUILD_SUPERUSER)
+unsub_map = on_command('unsubmap', aliases = {'取消订阅地图'}, permission = SUPERUSER | GROUP_ADMIN | GROUP_OWNER | GUILD_ADMIN | GUILD_OWNER | GUILD_SUPERUSER)
+sub_craft = on_command('subcraft', aliases = {'订阅制造'}, permission = SUPERUSER | GROUP_ADMIN | GROUP_OWNER | GUILD_ADMIN | GUILD_OWNER | GUILD_SUPERUSER)
+unsub_craft = on_command('unsubcraft', aliases = {'取消订阅制造'}, permission = SUPERUSER | GROUP_ADMIN | GROUP_OWNER | GUILD_ADMIN | GUILD_OWNER | GUILD_SUPERUSER)
+bind_uid = on_command('bind', aliases = {'绑定'})
+unbind_uid = on_command('unbind', aliases = {'解绑'})
+
+# Bot 连接
+@get_driver().on_bot_connect
+async def bot_connect():
+    await sql().on_start()
+    await job().on_start()
+
+# Bot 断开
+@get_driver().on_bot_disconnect
+async def bot_disconnect():
+    await sql().on_close()
 
 # 玩家名称查询
 @player_statistics.handle()
-async def _(bot: Bot, event: Union[PrivateMessageEvent, GroupMessageEvent], player_name: Message = CommandArg()):
+async def player_func(bot: Bot, event: Union[PrivateMessageEvent, GroupMessageEvent, GuildMessageEvent], player_name: Message = CommandArg()):
+    if not player_name:
+        await player_statistics.send('用法: /玩家 [玩家名称]')
+        return
     service = 'bridge'
     payload = {'auth': api_key, 'player': str(player_name), 'platform': 'PC'}
-    await player_statistics.send('正在查询: 玩家 {}'.format(player_name))
+    await player_statistics.send(f'正在查询: 玩家 {player_name}')
     response = await api_query(service, payload)
     if api_t2i:
-        msg = t2i(service, response)
+        msg = await t2i(service, response)
     else:
         msg = response
     await player_statistics.send(msg)
 
 # 玩家 UID 查询
 @uid_statistics.handle()
-async def _(bot: Bot, event: Union[PrivateMessageEvent, GroupMessageEvent], player_name: Message = CommandArg()):
+async def uid_func(bot: Bot, event: Union[PrivateMessageEvent, GroupMessageEvent, GuildMessageEvent], player_name: Message = CommandArg()):
+    if not player_name:
+        if await sql().check_uid(event.user_id):
+            player_name = await sql().check_uid(event.user_id)
+        else:
+            await player_statistics.send('用法: /UID [玩家 UID]')
+            return
     service = 'bridge'
     payload = {'auth': api_key, 'uid': str(player_name), 'platform': 'PC'}
-    await uid_statistics.send('正在查询: UID {}'.format(player_name))
+    await uid_statistics.send(f'正在查询: UID {player_name}')
     response = await api_query(service, payload)
     if api_t2i:
-        msg = t2i(service, response)
+        msg = await t2i(service, response)
     else:
         msg = response
     await uid_statistics.send(msg)
 
 # 地图轮换查询
 @map_protation.handle()
-async def _(bot: Bot, event: Union[PrivateMessageEvent, GroupMessageEvent]):
+async def map_func(bot: Bot, event: Union[PrivateMessageEvent, GroupMessageEvent, GuildMessageEvent]):
     service = 'maprotation'
     payload = {'auth': api_key, 'version': '2'}
     await map_protation.send('正在查询: 地图轮换')
     response = await api_query(service, payload)
     if api_t2i:
-        msg = t2i(service, response)
+        msg = await t2i(service, response)
     else:
         msg = response
     await map_protation.send(msg)
 
 # 顶尖猎杀者查询
 @predator.handle()
-async def _(bot: Bot, event: Union[PrivateMessageEvent, GroupMessageEvent]):
+async def predator_func(bot: Bot, event: Union[PrivateMessageEvent, GroupMessageEvent, GuildMessageEvent]):
     service = 'predator'
     payload = {'auth': api_key}
     await predator.send('正在查询: 顶尖猎杀者')
     response = await api_query(service, payload)
     if api_t2i:
-        msg = t2i(service, response)
+        msg = await t2i(service, response)
     else:
         msg = response
     await predator.send(msg)
 
 # 制造轮换查询
 @crafting_rotation.handle()
-async def _(bot: Bot, event: Union[PrivateMessageEvent, GroupMessageEvent]):
+async def crafting_func(bot: Bot, event: Union[PrivateMessageEvent, GroupMessageEvent, GuildMessageEvent]):
     service = 'crafting'
     payload = {'auth': api_key}
     await crafting_rotation.send('正在查询: 制造轮换')
     response = await api_query(service, payload)
     if api_t2i:
-        msg = t2i(service, response)
+        msg = await t2i(service, response)
     else:
         msg = response
     await crafting_rotation.send(msg)
 
 # 服务器状态查询
 @servers.handle()
-async def _(bot: Bot, event: Union[PrivateMessageEvent, GroupMessageEvent]):
+async def servers_func(bot: Bot, event: Union[PrivateMessageEvent, GroupMessageEvent, GuildMessageEvent]):
     service = 'servers'
     payload = {'auth': api_key}
     await servers.send('正在查询: 服务器状态')
@@ -125,73 +159,96 @@ async def _(bot: Bot, event: Union[PrivateMessageEvent, GroupMessageEvent]):
 
 # 订阅地图轮换
 @sub_map.handle()
-async def _(bot: Bot, event: GroupMessageEvent):
+async def submap_func(bot: Bot, event: Union[GroupMessageEvent, GuildMessageEvent]):
     try:
-        scheduler.add_job(func=submap, trigger='cron', id=(str(event.group_id) + '_map'), minute=1, kwargs={'bot': bot, 'group_id': event.group_id, 'api_t2i': api_t2i})
-        await sub_map.send('已订阅地图轮换')
-    except BaseException as err:
-        await sub_map.send('订阅地图轮换失败: {}'.format(err))
+        hour = None
+        bot_id = bot.self_id
+        if isinstance(event, GroupMessageEvent):
+            id = str(event.group_id) + '_map'
+            await job().add(submap, id, hour, bot_id, group_id = event.group_id)
+        elif isinstance(event, GuildMessageEvent):
+            id = str(event.channel_id) + '_map'
+            await job().add(submap, id, hour, bot_id, guild_id = event.guild_id, channel_id = event.channel_id)
+        await sql().addsub(event, bot_id, 'maprotation')
+        await sub_map.send('已订阅地图轮换。')
+    except Exception as err:
+        await sub_map.send(f'订阅地图轮换失败。\n{err}')
 
 # 地图轮换定时任务
-async def submap(bot, group_id, api_t2i):
+async def submap(bot_id, group_id, guild_id, channel_id):
+    bot = get_bot(bot_id)
     service = 'maprotation'
     payload = {'auth': api_key, 'version': '2'}
-    await bot.send_group_msg(group_id=group_id, message='正在查询: 地图轮换')
     response = await api_query(service, payload)
     if api_t2i:
-        title = '地图轮换'
-        font_size = 32
-        text = response
-        Txt2Img().set_font_size(font_size)
-        pic = Txt2Img().draw(title, text)
-        msg = MessageSegment.image(pic)
+        msg = await t2i(service, response)
     else:
         msg = response
-    await bot.send_group_msg(group_id=group_id, message=msg)
+    if group_id:
+        await bot.send_group_msg(group_id = group_id, message = msg)
+    elif channel_id:
+        await bot.send_guild_channel_msg(guild_id = guild_id, channel_id = channel_id, message = msg)
 
 # 取消订阅地图轮换
 @unsub_map.handle()
-async def _(bot: Bot, event: GroupMessageEvent):
+async def unsub_map_func(bot: Bot, event: Union[GroupMessageEvent, GuildMessageEvent]):
     try:
-        scheduler.remove_job(job_id=(str(event.group_id) + '_map'))
-        await unsub_map.send('已取消订阅地图轮换')
-    except BaseException as err:
-        await unsub_map.send('取消订阅地图轮换失败: {}'.format(err))
+        if isinstance(event, GroupMessageEvent):
+            id = str(event.group_id) + '_map'
+        elif isinstance(event, GuildMessageEvent):
+            id = str(event.channel_id) + '_map'
+        await job().remove(id)
+        await sql().delsub(event, 'maprotation')
+        await unsub_map.send('已取消订阅地图轮换。')
+    except Exception as err:
+        await unsub_map.send(f'取消订阅地图轮换失败。\n{err}')
 
 # 订阅制造轮换
 @sub_craft.handle()
-async def _(bot: Bot, event: GroupMessageEvent):
+async def subcraft_func(bot: Bot, event: Union[GroupMessageEvent, GuildMessageEvent]):
     try:
-        scheduler.add_job(func=subcraft, trigger='cron', id=(str(event.group_id) + '_craft'), hour=2, minute=1, kwargs={'bot': bot, 'group_id': event.group_id, 'api_t2i': api_t2i})
-        await sub_craft.send('已订阅制造轮换')
-    except BaseException as err:
-        await sub_craft.send('订阅制造轮换失败: {}'.format(err))
+        hour = 2
+        bot_id = bot.self_id
+        if isinstance(event, GroupMessageEvent):
+            id = str(event.group_id) + '_craft'
+            await job().add(subcraft, id, hour, bot_id, group_id = event.group_id)
+        elif isinstance(event, GuildMessageEvent):
+            id = str(event.channel_id) + '_craft'
+            await job().add(subcraft, id, hour, bot_id, guild_id = event.guild_id, channel_id = event.channel_id)
+        await sql().addsub(event, bot_id, 'crafting')
+        await sub_craft.send('已订阅制造轮换。')
+    except Exception as err:
+        await sub_craft.send(f'订阅制造轮换失败。\n{err}')
+
 
 # 制造轮换定时任务
-async def subcraft(bot, group_id, api_t2i):
+async def subcraft(bot_id, group_id, guild_id, channel_id):
+    bot = get_bot(bot_id)
     service = 'crafting'
     payload = {'auth': api_key}
-    await bot.send_group_msg(group_id=group_id, message='正在查询: 制造轮换')
     response = await api_query(service, payload)
     if api_t2i:
-        title = '制造轮换'
-        font_size = 32
-        text = response
-        Txt2Img().set_font_size(font_size)
-        pic = Txt2Img().draw(title, text)
-        msg = MessageSegment.image(pic)
+        msg = await t2i(service, response)
     else:
         msg = response
-    await bot.send_group_msg(group_id=group_id, message=msg)
+    if group_id:
+        await bot.send_group_msg(group_id = group_id, message = msg)
+    elif channel_id:
+        await bot.send_guild_channel_msg(guild_id = guild_id, channel_id = channel_id, message = msg)
 
 # 取消订阅制造轮换
 @unsub_craft.handle()
-async def _(bot: Bot, event: GroupMessageEvent):
+async def unsub_craft_func(bot: Bot, event: Union[GroupMessageEvent, GuildMessageEvent]):
     try:
-        scheduler.remove_job(job_id=(str(event.group_id) + '_craft'))
-        await unsub_craft.send('已取消订阅制造轮换')
-    except BaseException as err:
-        await unsub_craft.send('取消订阅制造轮换失败: {}'.format(err))
+        if isinstance(event, GroupMessageEvent):
+            id = str(event.group_id) + '_craft'
+        elif isinstance(event, GuildMessageEvent):
+            id = str(event.channel_id) + "_craft"
+        await job().remove(id)
+        await sql().delsub(event, 'crafting')
+        await unsub_craft.send(f'已取消订阅制造轮换。')
+    except Exception as err:
+        await unsub_craft.send(f'取消订阅制造轮换失败。\n{err}')
 
 # 异步查询
 async def api_query(service, payload):
@@ -199,16 +256,16 @@ async def api_query(service, payload):
         async with AsyncClient() as client:
             response = await client.get(api_url + service, params = payload, timeout = None)
         if response.status_code != 200 or response.text.find('Error') != -1:
-            data = '查询失败: API 错误: {}'.format(response.text)
+            data = f'查询失败: API 错误: {response.text}'
         else:
             data = process(service, response)
         return data
-    except BaseException as err:
-        data = '查询失败: 网络错误: {}'.format(err)
+    except Exception as err:
+        data = f'查询失败: 网络错误: {err}'
         return data
 
 # txt2img
-def t2i(service, response):
+async def t2i(service, response):
     title = convert(service)
     font_size = 32
     text = response
@@ -217,6 +274,131 @@ def t2i(service, response):
     msg = MessageSegment.image(pic)
     return msg
 
+# 绑定 UID
+@bind_uid.handle()
+async def bind_uid_func(bot: Bot, event: Union[PrivateMessageEvent, GroupMessageEvent, GuildMessageEvent], uid: Message = CommandArg()):
+    if uid:
+        try:
+            if isinstance(event, GuildMessageEvent):
+                await sql().adduid(event, uid)
+                await bind_uid.send(f'已将 UID {uid} 绑定至 频道用户 {event.sender.nickname} 。')
+            else:
+                await sql().adduid(event, uid)
+                await bind_uid.send(f'已将 UID {uid} 绑定至 QQ {event.user_id} 。')
+        except Exception as err:
+            await bind_uid.send(f'绑定失败。\n{err}')
+    else: 
+        await bind_uid.send('用法: /绑定 [玩家 UID]')
+
+# 解绑 UID
+@unbind_uid.handle()
+async def unbind_uid_func(bot: Bot, event: Union[PrivateMessageEvent, GroupMessageEvent, GuildMessageEvent]):
+    try:
+        if isinstance(event, GuildMessageEvent):
+            await sql().deluid(event)
+            await unbind_uid.send(f'已将 频道用户 {event.user_id} 解绑。')
+        else:
+            await sql().deluid(event)
+            await unbind_uid.send(f'已将 QQ {event.user_id} 解绑')
+    except Exception as err:
+        await unbind_uid.send(f'解绑失败。\n{err}')
+
+# 数据库操作
+class sql:
+
+    # 连接数据库
+    conn = sqlite3.connect(plugin_data_file)
+    c = conn.cursor()
+
+    # 启动时创建数据库
+    async def on_start(self):
+        self.c.execute('CREATE TABLE IF NOT EXISTS "ID" ("QQ" TEXT, "EA" TEXT, PRIMARY KEY ("QQ"))')
+        self.c.execute('CREATE TABLE IF NOT EXISTS "SUB" ("ID" TEXT, "Guild_ID" TEXT, "Craft" INTEGER, "Map" INTEGER, "Bot_ID" TEXT NOT NULL, PRIMARY KEY ("ID"))')
+        self.conn.commit()
+
+    # 关闭时关闭数据库
+    async def on_close(self):
+        self.conn.close()
+
+    # 添加绑定信息
+    async def adduid(self, event, uid):
+        self.c.execute(f'INSERT INTO "ID" ("QQ", "EA") VALUES ("{event.user_id}", "{uid}") ON CONFLICT ("QQ") DO UPDATE SET "EA" = "{uid}"')
+        self.conn.commit()
+
+    # 删除绑定信息
+    async def deluid(self, event):
+        self.c.execute(f'DELETE FROM "ID" WHERE "QQ" = "{event.user_id}"')
+        self.conn.commit()
+
+    # 添加订阅信息
+    async def addsub(self, event, bot_id, service):
+        if isinstance(event, GroupMessageEvent):
+            if service == 'maprotation':
+                self.c.execute(f'INSERT INTO "SUB" ("ID", "Map", "Bot_ID") VALUES ("{event.group_id}", "1", "{bot_id}") ON CONFLICT ("ID") DO UPDATE SET "Map" = 1')
+            elif service == 'crafting':
+                self.c.execute(f'INSERT INTO "SUB" ("ID", "Craft", "Bot_ID") VALUES ("{event.group_id}", "1", "{bot_id}") ON CONFLICT ("ID") DO UPDATE SET "Craft" = 1')
+        elif isinstance(event, GuildMessageEvent):
+            if service == 'maprotation':
+                self.c.execute(f'INSERT INTO "SUB" ("ID", "Guild_ID", "Map", "Bot_ID") VALUES ("{event.channel_id}", "{event.guild_id}", "1", "{bot_id}") ON CONFLICT ("ID") DO UPDATE SET "Map" = 1')
+            elif service == 'crafting':
+                self.c.execute(f'INSERT INTO "SUB" ("ID", "Guild_ID", "Craft", "Bot_ID") VALUES ("{event.channel_id}", "{event.guild_id}", "1", "{bot_id}") ON CONFLICT ("ID") DO UPDATE SET "Craft" = 1')
+        self.c.execute
+        self.conn.commit()
+
+    # 修改订阅信息
+    async def delsub(self, event, service):
+        if isinstance(event, GroupMessageEvent):
+            if service == 'maprotation':
+                self.c.execute(f'UPDATE "SUB" SET "Map" = 0 WHERE "ID" = "{event.group_id}"')
+            elif service == 'crafting':
+                self.c.execute(f'UPDATE "SUB" SET "Craft" = 0 WHERE "ID" = "{event.group_id}"')
+        elif isinstance(event, GuildMessageEvent):
+            if service == 'maprotation':
+                self.c.execute(f'UPDATE "SUB" SET "Map" = 0 WHERE "ID" = "{event.channel_id}"')
+            elif service == 'crafting':
+                self.c.execute(f'UPDATE "SUB" SET "Craft" = 0 WHERE "ID" = "{event.channel_id}"')
+        self.conn.commit()
+
+    # 查询 UID
+    async def check_uid(self, user_id):
+        cursor = self.c.execute(f'SELECT "QQ", "EA" FROM "ID" WHERE "QQ" = "{user_id}"')
+        return cursor.fetchone()[1]
+
+    # 查询订阅信息
+    async def get_sub(self):
+        cursor = self.c.execute('SELECT * FROM "SUB"')
+        return cursor.fetchall()
+
+# 定时任务操作
+class job:
+
+    # 添加定时任务
+    async def add(self, func, id, hour, bot_id, group_id = None, guild_id = None, channel_id = None):
+        scheduler.add_job(func=func, trigger='cron', id=id, hour=hour, minute=1, kwargs={'bot_id': bot_id, 'group_id': group_id, 'guild_id': guild_id, 'channel_id': channel_id})
+
+    # 删除定时任务
+    async def remove(self, id):
+        scheduler.remove_job(job_id=id)
+    
+    # 自动添加定时任务
+    async def on_start(self):
+        cursor = await sql().get_sub()
+        for row in cursor:
+            ID = row[0]
+            Guild_ID = row[1]
+            Craft = bool(row[2])
+            Map = bool(row[3])
+            Bot_ID = row[4]
+            if Craft:
+                if Guild_ID:
+                    await job().add(func=subcraft, id=(ID + '_craft'), hour=2, bot_id=Bot_ID, guild_id=Guild_ID, channel_id=ID)
+                elif not Guild_ID:
+                    await job().add(func=subcraft, id=(ID + '_craft'), hour=2, bot_id=Bot_ID, group_id=ID)
+            if Map:
+                if Guild_ID:
+                    await job().add(func=submap, id=(ID + '_map'), hour=None, bot_id=Bot_ID, guild_id=Guild_ID, channel_id=ID)
+                elif not Guild_ID:
+                    await job().add(func=submap, id=(ID + '_map'), hour=None, bot_id=Bot_ID, group_id=ID)
 
 # 处理获取信息
 def process(service, response):
@@ -283,7 +465,7 @@ def process(service, response):
             '当前地图: {}\n'
             '下个地图: {}\n'
             '剩余时间: {}\n\n'
-            '限时:\n'
+            '混录带:\n'
             '当前地图: {}\n'
             '下个地图: {}\n'
             '剩余时间: {}'
@@ -486,7 +668,11 @@ def convert(name):
         'Phase runner': '相位穿梭器',
         'Party crasher': '派对破坏者',
         'Drop Off': '原料场',
-        'Skull Town': '骷髅镇',
+        'Skulltown': '骷髅镇',
+        'Barometer': '气压计',
+        'Wall': '高墙',
+        'Siphon': '岩浆汲取器',
+        'Fragment': '碎片东部',
         'Unknown': '未知',
         'None': '无',
         # Barrels
